#include <LiquidCrystal.h>

#include <DFRobot_RGBLCD.h>
#include <Wire.h>
#include "DFRobot_RGBLCD.h"

DFRobot_RGBLCD lcd(16, 2);                 // set the RGB LCD called lcd

#define A 3                                // Encoder I02 Input pin 3
#define B 4                                // Encoder I01 Input pin 4
#define Button  12                         // Button input pin 12
#define D 80                               // Diameter 80mm

float C = 0;                               // perimeter (will be calculated)
unsigned int Distance;                     // total distance
int VA = 0;                                // Measurement Direction 1
int VB = 0;                                // Measurement Direction 2
unsigned long Count = 0;                   // positive count
unsigned int Count_1 = 0;                  // Negative count
unsigned char Mode = 0;                    // Current input mode
unsigned long lasttime = 0;                // Time since pressed
unsigned long Modetime = 0;                // Time in Mode
long buttonTimer = 0;                      // Time pressed
long longPressTime = 250;                  // Predefined time pressed
boolean buttonActive = false;              // Button short press logic
boolean longPressActive = false;           // Button long press logic
int unit = 0;                              // Unit:0 is cm    Unit:1 is in
int light = 0;                             // Light:0 is ON   Light:1 is OFF

void setup(){
  Serial.begin(9600);                       // start serial moniter if needed
  lcd.init();                               // initialize lcd
  lcd.setRGB(40, 153, 255);                 // set initial color of RGB lcd
  lcd.setCursor(2, 0 );                     // set the start point for printing
  lcd.print("M:");                          // print "M" for measure
  lcd.setCursor(2, 1 );                     // set the point for printing on the line below "M" 
  lcd.print("D:");                          // print "D" for Distance
  pinMode(A, INPUT_PULLUP);                 // Pull-up input for Encoder
  pinMode(B, INPUT_PULLUP);                 // Pull-up input for Encoder
  pinMode(Button, INPUT);                   // Button Input   
  attachInterrupt(1, interrupt, RISING);    // make an interupt on a rising input
  C = D * PI;                               // Calculate Circumfrence with D=80mm
}


void loop(){

  
   if (millis() - 3000 > lasttime){         // Detect Buttons once every 3s
    
    if (digitalRead(Button) == HIGH)        // If Button is HIGH  
      if (digitalRead(Button) == HIGH)
        if (unit == 0){                     // If unit is cm
          unit = 1;                         // Set unit to in
          }
     else if (unit == 1)                    // If unit is in              
          unit = 0;                         // Set unit to cm           

    while (digitalRead(Button) == HIGH);    // while button is pressed
    lasttime = millis();                    // set a time for time held
  }

  

     
if (millis() - 150 > lasttime )             // Detect Buttons once every 150ms
  {
    if (digitalRead(Button) == HIGH)        // If Button is HIGH
      if (digitalRead(Button) == HIGH)
        Mode += 1;                          // Increase Mode by 1
    if (Mode > 2)                           // if the mode is greater than 2
      Mode = 0;                             // reset mode to 0

    while (digitalRead(Button) == HIGH);    // while button is pressed
    lasttime = millis();                    // set a time for time held
  }

  else if (millis() - 50 > Modetime)        // Refresh the data every 50ms
  {

      if(millis() - 3000 > Modetime){       // If button held for 3 seconds
        
        if (light == 0){                    // If light is at 0 meaning LCD is on
           lcd.noDisplay();                 // Turn LCD off
           lcd.setRGB(0,0,0);               // set backlight to 0,0,0 which is black and off
           light = 1;                       // set light to 1 which means lcd is off
     }
        else if (light == 1){               // if light is 1 meaning lcd is off
           lcd.display();                   // turn on lcd display
           light = 0;                       // set light to 0 meaning the lcd is on
     } 
    }


    else if(millis() - 500 > Modetime ){    // if held for 2 seconds or more
       if (unit == 0 && Mode == 0){         // if unit in Cm mode
          unit = 1;                         // set units to inches
          lcd.setCursor(12, 1 );            // set the point for printing on the same line but towards the end
          lcd.print("in");                  // print "in" for centimeters
          }
       else if (unit == 1 && Mode == 0){    // if unit in inches      
          unit = 0;                         // set unit to cm mode 
          lcd.setCursor(12, 1 );            // set the point for printing on the same line but towards the end
          lcd.print("cm");                  // print "cm" for centimeters  
     }
    }
   
    
    if (Mode == 0 && light==0)              // if Mode is 0 then it is on Reset
    {
      lcd.setRGB(40, 153, 255);             // set RGB Color to Green
      lcd.setCursor(6, 0 );                 // set cursor position
      lcd.print("Reset    ");               // Print "Reset" for mode
      Distance = C * Count / 40;            // Calculate the distance based of rotations
      lcd.setCursor(11, 0 );
      lcd.print("     ");
    }




    if (Mode == 1 && light==0)              // Mode is on Measure: Calculate the measured value
    {
      lcd.setRGB(50, 255, 50);              // set RGB Color to a Blue Green
      lcd.setCursor(6, 0 );                 // set cursor position
      lcd.print("Measure");                 // Print "Mesasure" for mode
      lcd.setCursor(4, 1 );                 // set cursor position for measuring

      
      if (unit == 0){                       // if in unit being 0 meaning CM mode
      lcd.setCursor(12, 1 );                // set the point for printing on the same line but towards the end
      lcd.print("cm");                      // print "cm" for centimeters
      lcd.setCursor(4, 1 );
 
      if (Count > 0 && Count < 0xffff)      // Determine whether the length is positive
      {
        lcd.print('-');                     // The length is negative Print "-"
        Distance = C * Count / 40;          // Calculate Distance
      }


      else if (Count == 0 && Count_1 == 0 ) // Determine whether the length is zero
      {
        lcd.setCursor(4, 1 );               // set cursor position
        lcd.print(' ');                     // Print a blank space
        Distance = C * Count / 40;          // Calculate Distance
      }


      else                                  // Length is positive
      {
        lcd.print('+');                     // The length is Positive Print "+"
        Distance = Count_1 * C  / 40;       // Calculate Distance with positive count_1
      }
    }
    

 if (unit == 1){                            // if units in at 1 meaning inches
      lcd.setCursor(12, 1 );                // set the point for printing on the same line but towards the end
      lcd.print("in");                      // print "in" for centimeters
      lcd.setCursor(4, 1 );
 
      if (Count > 0 && Count < 0xffff)      // Determine whether the length is positive
      {
        lcd.print('-');                     // The length is negative Print "-"
        Distance = C * Count / (40*2.54);   // Calculate Distance
      }


      else if (Count == 0 && Count_1 == 0 ) // Determine whether the length is zero
      {
        lcd.setCursor(4, 1 );               // set cursor position
        lcd.print(' ');                     // Print a blank space
        Distance = C * Count / (40*2.54);   // Calculate Distance
      }


      else                                  // Length is positive
      {
        lcd.print('+');                     // The length is Positive Print "+"
        Distance = Count_1 * C  / (40*2.54);// Calculate Distance with positive count_1
      }
    }
  }

    else if (Mode == 2 && light==0)         // If in lock mode
    {
      lcd.setRGB(255, 51, 51);              // set RGB Color to RED pink
      lcd.setCursor(6, 0 );                 // set cursor position
      lcd.print("Lock     ");               // Print lock with a space for the distance
      Count = 0;                            // Reset Counts
      Count_1 = 0;                          // Reset Counts
    }

    Modetime = millis();                    // units of modetime
  }

  lcd.setCursor(5, 1 );                     // Set cursor position for display distance
  lcd.print(Distance / 10000);              // Displays the value of Distance "100"
  lcd.print((Distance / 1000) % 10);        // Displays the value of Distance "10"
  lcd.print((Distance / 100) % 10);         // Displays the value of Distance "1"
  lcd.print('.');                           // Displays the value of Distance "decimal"
  lcd.print((Distance / 10) % 10);          // Displays the value of Distance ".1"
  lcd.print(Distance % 10);                 // Displays the value of Distance ".01"
}

void interrupt()                              // Interrupt handler
{
  VB = digitalRead(B);
  if (Mode == 1)                              // Detects whether the current mode is a measurement
  {
    if (VB == 1)                              // To determine whether the positive measurement
    {
      
      if (Count > 0xffff)                     // if above max measurment and count is positive 
      {
        Count_1 -= 1;                         // subtract from second count                     
      }
      Count += 1;                             // increase the count
      }
    
    else                                      // Reverse measurement
    {
      Count -= 1;                             // subtract from count
 
    if (Count > 0xFFFF)                       // above max and count negative                      
      {
        Count_1 += 1;                         // increase count
      }
    }
    
   
    if (Count < 0xFFFF && Count > 0x294A)      // Count between 10,570 and 65,535     (Close to negative max)
      Count = 0;                               // reset count
 
    else if (Count < 0xD6B5 && Count > 0xFFFF) // count between 54,965 and 65,535     (Close to positive max)
      Count = 0;                               // reset count
  }
}
